
// color to be used for procedural textures
#define PROCCOLOR "#(argb,8,8,3)color(1,0,0,1)"

// debug outputs
//#define DEBUG


#include "defines.hpp"
#include "functions.hpp"

// clear any remaining intro message
titleText ["","plain"];

// find the unit player is looking at
_men = nearestObjects [player,["AllVehicles"],50];
_man = objNull;
{
	if (!(player==_x) && (player isLookingAt [_x,90])) exitWith {
		_man = _x;
	};
}forEach _men;
if (isNull _man) exitWith {
	warningMessage "Get close to a unit, to interact with its selections and textures."
};



// create the dialog, and adjust for different aspect ratios
createDialog "Dlg_Dressup";
allowMovementControlsInDialog true;

_display  = findDisplay IDD_DLG_DRESSUP;
// adjust dialog for aspect ratio
[_display] call _adjustDialog;

_unitTxtCtrl = _display displayCtrl IDC_UNITTYPE_TXT;
_selLBCtrl = _display displayCtrl IDC_SELECTIONS_LB;
_texLBCtrl = _display displayCtrl IDC_TEXTURES_LB;
_pasteTxtCtrl  = _display displayCtrl IDC_PASTE_EDT;

// get information about the currently selected unit
_mantype = getText (configFile >> "cfgVehicles" >> typeof _man >> "displayName");
_manclass = getText (configFile >> "cfgVehicles" >> typeof _man >> "vehicleClass");
_manmodel = getText (configFile >> "cfgVehicles" >> typeof _man >> "model");
_unitTxtCtrl ctrlSetText _mantype;
#ifdef DEBUG
echo format["[K] Examining '%1 -%2 (%3)':",_mantype,_manclass,_manmodel];
#endif

// find other units that use the same model
_samemodel = format["(getText (_x >> 'model'))=='%1'",_manmodel] configClasses (configFile >> "CfgVehicles");
#ifdef DEBUG
echo format["[K] Units using same model: %1",count _samemodel];
#endif

_selections = [];
_textures = [];
{
	_xType = getText (_x >> "displayName");
	_xSelections = getArray (_x >> "hiddenSelections");
	_xTextures0 = getArray (_x >> "hiddenSelectionsRandomTextures");
	_xTextures1 = getArray (_x >> "hiddenSelectionsTextures");
	
	#ifdef DEBUG
	echo format["[K] "];
	echo format["[K] %1",configName _x];
	echo format["[K] Selections: %1",_xSelections];
	echo format["[K] RandomTextures0: %1",_xTextures0];
	echo format["[K] HiddenTextures1: %1",_xTextures1];
	#endif
	
	// go through all selections that are defined for this type
	for "_i" from 0 to (count _xSelections)-1 do {
		_selection = toLower(_xSelections select _i);
		// check if the current selection already exists in the overall selection array
		// if not, add it
		if !(_selection in _selections) then {
			_selections = _selections + [_selection];
		};
		// get the index of the current selection for the overall array
		_selidx = _selections find _selection;
		#ifdef DEBUG
		echo format["[K] _selection: %1, %2",_selidx,_selection];
		#endif
		// get the so far accumulated textures for this selection
		_selTextures = [];
		if (count _textures>=_selidx+1) then {
			_selTextures = _textures select _selidx;
		};
		#ifdef DEBUG
		echo format["[K] _selTextures: %1",_selTextures];
		#endif
		
		// find out which textures are available for the evaluated selection
		_xTextures = [];
		// random textures: nested arrays with several entries for each selection
		if (count _xTextures0>=_selidx+1) then {
			_randomTextures = _xTextures0 select _selidx;
			#ifdef DEBUG
			echo format["[K] random: %1, %2",_selidx,_randomTextures];
			#endif
			{
				_xTextures = _xTextures + [_x];
			}forEach _randomTextures;
		};
		#ifdef DEBUG
		echo format["[K] _xTextures: %1",_xTextures];
		#endif
		// regular textures, one array element per selection
		if (count _xTextures1>=_selidx+1) then {
			_xTextures = _xTextures + [_xTextures1 select _selidx];
		};
		// if we do have textures that match the selection, then add them to the overall array
		{
			// don't add procedural or empty textures
			if (!("#" in _x) && (_x!="") && !(_x in _selTextures)) then {
				_selTextures = _selTextures + [_x];
			};
		}forEach _xTextures;
		#ifdef DEBUG
		echo format["[K] _selTextures: %1, %2",_selTextures,_selidx];
		#endif
		// update the overall texture array with the updated ones
		_textures set [_selidx,_selTextures];

		#ifdef DEBUG
		echo format["[K] "];
		echo format["[K] _selections: %1",_selections];
		echo format["[K] _textures: %1",_textures];	
		echo format["[K] "];
		#endif
	};	
}forEach _samemodel;


// fill selection listbox
{
	_selLBCtrl lbAdd _x;
}forEach _selections;

dressup="";
_lastidx1 = -1;
_selection = "";
_lastselection = "";
_lastidx2 = -1;
_changed = false;

while {dialog} do {
	_selidx1 = lbCurSel _selLBCtrl;
	_lastselection = _selection;
	_selection = _selLBCtrl lbText _selidx1;
	
	// selection has changed
	if (_selidx1!=_lastidx1) then {
		// if last texture wasn't changed manually, reset to default one (to get rid of the procedural texture)
		if !(_changed) then {
			_man setObjectTexture [_lastselection,"#reset"];
		};
		_lastidx1 = _selidx1;
		lbClear _texLBCtrl;
		// add three special textures
		_texLBCtrl lbAdd "[none]"; // texture=""
		_texLBCtrl lbAdd "[default]"; // texture="#reset"
		_texLBCtrl lbAdd "[procedural texture]"; // texture=PROCCOLOR (define in header)
		// find out which textures match the new selection
		_selTextures = [];
		{
			_idx = _texLBCtrl lbAdd _x;
			_selTextures = _selTextures + [_x];
		}forEach (_textures select _selidx1);
		// put those textures into the second listbox
		{
			_seltext = _x;
			{
				if !(_x in _selTextures) then {
					_texLBCtrl lbAdd format["(%1)",_x];
					_selTextures = _selTextures + [_x];
				};
			}forEach _seltext;
		}forEach _textures;
		_lastidx2 = -1;
		// switch to procedural texture by default (to highlight it)
		_texLBCtrl lbSetCurSel 2;
	};

	_selidx2 = lbCurSel _texLBCtrl;
	// selected texture has changed
	if (_selidx2!=_lastidx2) then {
		// ignore default change to procedural texture
		_changed = (_lastidx2!=-1);
		_lastidx2 = _selidx2;
		_texture = "";
		if (_selidx2>0) then {
			// special texture selection [default]
			_texture = "#reset";
			if (_selidx2>1) then {
				// special texture selection [procedural color]
				_texture = PROCCOLOR;
				if (_selidx2>2) then {
					// otherwise use the regular texture that was selected
					_texture = _texLBCtrl lbText _selidx2;
					if (trim [_texture,0,(strlen _texture)-1]=="(") then {
						_texture = trim[_texture,1,1];
					};
				};
			};
		};
		// apply the texture
		_man setObjectTexture [_selection,_texture];
		// and fill the paste box with the respective command
		_pasteTxtCtrl ctrlSetText format["_man setObjectTexture ['%1','%2']",_selection,_texture]; 
		// display warning message for textures in parentheses

	};

	// reset to default textures
	if (dressup=="reset") then {
		dressup="";
		{
			_man setObjectTexture [_x,"#reset"];
		}forEach _selections;
	};
	
	// apply command from edit field
	if (dressup=="apply") then {
		_command = toLower(ctrlText _pasteTxtCtrl);
		if ("setobjecttexture" in _command) then {
			dressup="";
			call compile _command;
			_changed = true;
		};
	};
	
	sleep .1;
};

// reset procudural texture if dialog was escaped
if !(_changed) then {
	_man setObjectTexture [_lastselection,"#reset"];
};
